package sun.net.www;

import java.nio.charset.CharacterCodingException;
import java.net.URISyntaxException;
import java.net.URI;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CodingErrorAction;
import sun.nio.cs.ThreadLocalCoders;
import java.nio.CharBuffer;
import java.nio.ByteBuffer;
import java.io.File;
import java.util.BitSet;

public class ParseUtil
{
    static BitSet encodedInPath;
    private static final char[] hexDigits;
    private static final long L_DIGIT;
    private static final long H_DIGIT = 0L;
    private static final long L_HEX;
    private static final long H_HEX;
    private static final long L_UPALPHA = 0L;
    private static final long H_UPALPHA;
    private static final long L_LOWALPHA = 0L;
    private static final long H_LOWALPHA;
    private static final long L_ALPHA = 0L;
    private static final long H_ALPHA;
    private static final long L_ALPHANUM;
    private static final long H_ALPHANUM;
    private static final long L_MARK;
    private static final long H_MARK;
    private static final long L_UNRESERVED;
    private static final long H_UNRESERVED;
    private static final long L_RESERVED;
    private static final long H_RESERVED;
    private static final long L_ESCAPED = 1L;
    private static final long H_ESCAPED = 0L;
    private static final long L_DASH;
    private static final long H_DASH;
    private static final long L_URIC;
    private static final long H_URIC;
    private static final long L_PCHAR;
    private static final long H_PCHAR;
    private static final long L_PATH;
    private static final long H_PATH;
    private static final long L_USERINFO;
    private static final long H_USERINFO;
    private static final long L_REG_NAME;
    private static final long H_REG_NAME;
    private static final long L_SERVER;
    private static final long H_SERVER;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    public ParseUtil() {
    }
    
    public static String encodePath(String s) {
        return encodePath(s, true);
    }
    
    public static String encodePath(String s, boolean b) {
        char[] value = new char[s.length() * 2 + 16];
        int count = 0;
        char[] charArray = s.toCharArray();
        for (int length = s.length(), i = 0; i < length; ++i) {
            int bitIndex = charArray[i];
            if ((!b && bitIndex == '/') || (b && bitIndex == File.separatorChar)) {
                value[count++] = '/';
            }
            else if (bitIndex <= '\u007f') {
                if ((bitIndex >= 'a' && bitIndex <= 'z') || (bitIndex >= 'A' && bitIndex <= 'Z') || (bitIndex >= '0' && bitIndex <= '9')) {
                    value[count++] = bitIndex;
                }
                else if (ParseUtil.encodedInPath.get(bitIndex)) {
                    count = escape(value, (char)bitIndex, count);
                }
                else {
                    value[count++] = bitIndex;
                }
            }
            else if (bitIndex > '߿') {
                count = escape(value, (char)(0x80 | (bitIndex >> 0 & 0x3F)), escape(value, (char)(0x80 | (bitIndex >> 6 & 0x3F)), escape(value, (char)(0xE0 | (bitIndex >> 12 & 0xF)), count)));
            }
            else {
                count = escape(value, (char)(0x80 | (bitIndex >> 0 & 0x3F)), escape(value, (char)(0xC0 | (bitIndex >> 6 & 0x1F)), count));
            }
            if (count + 9 > value.length) {
                int n = value.length * 2 + 16;
                if (n < 0) {
                    n = Integer.MAX_VALUE;
                }
                char[] array = new char[n];
                System.arraycopy(value, 0, array, 0, count);
                value = array;
            }
        }
        return new String(value, 0, count);
    }
    
    private static int escape(char[] array, char c, int n) {
        array[n++] = '%';
        array[n++] = Character.forDigit(c >> 4 & 0xF, 16);
        array[n++] = Character.forDigit(c & '\u000f', 16);
        return n;
    }
    
    private static byte unescape(String s, int n) {
        return (byte)Integer.parseInt(s.substring(n + 1, n + 3), 16);
    }
    
    public static String decode(String s) {
        int length = s.length();
        if (length == 0 || s.indexOf(37) < 0) {
            return s;
        }
        StringBuilder sb = new StringBuilder(length);
        ByteBuffer allocate = ByteBuffer.allocate(length);
        CharBuffer allocate2 = CharBuffer.allocate(length);
        CharsetDecoder onUnmappableCharacter = ThreadLocalCoders.decoderFor("UTF-8").onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT);
        int c = s.charAt(0);
        int i = 0;
    Label_0069:
        while (i < length) {
            assert c == s.charAt(i);
            if (c != '%') {
                sb.append((char)c);
                if (++i >= length) {
                    break;
                }
                c = s.charAt(i);
            }
            else {
                allocate.clear();
                while (ParseUtil.$assertionsDisabled || length - i >= 2) {
                    try {
                        allocate.put(unescape(s, i));
                    }
                    catch (NumberFormatException ex) {
                        throw new IllegalArgumentException();
                    }
                    i += 3;
                    if (i < length) {
                        c = s.charAt(i);
                        if (c == '%') {
                            continue;
                        }
                    }
                    allocate.flip();
                    allocate2.clear();
                    onUnmappableCharacter.reset();
                    if (onUnmappableCharacter.decode(allocate, allocate2, true).isError()) {
                        throw new IllegalArgumentException("Error decoding percent encoded characters");
                    }
                    if (onUnmappableCharacter.flush(allocate2).isError()) {
                        throw new IllegalArgumentException("Error decoding percent encoded characters");
                    }
                    sb.append(allocate2.flip().toString());
                    continue Label_0069;
                }
                throw new AssertionError();
            }
        }
        return sb.toString();
    }
    
    public String canonizeString(String s) {
        s.length();
        int index;
        while ((index = s.indexOf("/../")) >= 0) {
            int lastIndex;
            if ((lastIndex = s.lastIndexOf(47, index - 1)) >= 0) {
                s = s.substring(0, lastIndex) + s.substring(index + 3);
            }
            else {
                s = s.substring(index + 3);
            }
        }
        int index2;
        while ((index2 = s.indexOf("/./")) >= 0) {
            s = s.substring(0, index2) + s.substring(index2 + 2);
        }
        while (s.endsWith("/..")) {
            int index3 = s.indexOf("/..");
            int lastIndex2;
            if ((lastIndex2 = s.lastIndexOf(47, index3 - 1)) >= 0) {
                s = s.substring(0, lastIndex2 + 1);
            }
            else {
                s = s.substring(0, index3);
            }
        }
        if (s.endsWith("/.")) {
            s = s.substring(0, s.length() - 1);
            return s;
        }
        return s;
    }
    
    public static URL fileToEncodedURL(File file) throws MalformedURLException {
        String file2 = encodePath(file.getAbsolutePath());
        if (!file2.startsWith("/")) {
            file2 = "/" + file2;
        }
        if (!file2.endsWith("/") && file.isDirectory()) {
            file2 += "/";
        }
        return new URL("file", "", file2);
    }
    
    public static URI toURI(URL url) {
        String protocol = url.getProtocol();
        String s = url.getAuthority();
        String str = url.getPath();
        String query = url.getQuery();
        String ref = url.getRef();
        if (str != null && !str.startsWith("/")) {
            str = "/" + str;
        }
        if (s != null && s.endsWith(":-1")) {
            s = s.substring(0, s.length() - 3);
        }
        URI uri;
        try {
            uri = createURI(protocol, s, str, query, ref);
        }
        catch (URISyntaxException ex) {
            uri = null;
        }
        return uri;
    }
    
    private static URI createURI(String s, String s2, String s3, String s4, String s5) throws URISyntaxException {
        String string = toString(s, null, s2, null, null, -1, s3, s4, s5);
        checkPath(string, s, s3);
        return new URI(string);
    }
    
    private static String toString(String str, String s, String s2, String s3, String s4, int n, String s5, String s6, String s7) {
        StringBuffer sb = new StringBuffer();
        if (str != null) {
            sb.append(str);
            sb.append(':');
        }
        appendSchemeSpecificPart(sb, s, s2, s3, s4, n, s5, s6);
        appendFragment(sb, s7);
        return sb.toString();
    }
    
    private static void appendSchemeSpecificPart(StringBuffer sb, String substring, String s, String s2, String s3, int n, String s4, String s5) {
        if (substring != null) {
            if (substring.startsWith("//[")) {
                int index = substring.indexOf("]");
                if (index != -1 && substring.indexOf(":") != -1) {
                    String substring2;
                    if (index == substring.length()) {
                        substring2 = "";
                    }
                    else {
                        substring = substring.substring(0, index + 1);
                        substring2 = substring.substring(index + 1);
                    }
                    sb.append(substring);
                    sb.append(quote(substring2, ParseUtil.L_URIC, ParseUtil.H_URIC));
                }
            }
            else {
                sb.append(quote(substring, ParseUtil.L_URIC, ParseUtil.H_URIC));
            }
        }
        else {
            appendAuthority(sb, s, s2, s3, n);
            if (s4 != null) {
                sb.append(quote(s4, ParseUtil.L_PATH, ParseUtil.H_PATH));
            }
            if (s5 != null) {
                sb.append('?');
                sb.append(quote(s5, ParseUtil.L_URIC, ParseUtil.H_URIC));
            }
        }
    }
    
    private static void appendAuthority(StringBuffer sb, String substring, String s, String str, int i) {
        if (str != null) {
            sb.append("//");
            if (s != null) {
                sb.append(quote(s, ParseUtil.L_USERINFO, ParseUtil.H_USERINFO));
                sb.append('@');
            }
            int n = str.indexOf(58) >= 0 && !str.startsWith("[") && !str.endsWith("]");
            if (!(n == 0)) {
                sb.append('[');
            }
            sb.append(str);
            if (!(n == 0)) {
                sb.append(']');
            }
            if (i != -1) {
                sb.append(':');
                sb.append(i);
            }
        }
        else if (substring != null) {
            sb.append("//");
            if (substring.startsWith("[")) {
                int index = substring.indexOf("]");
                if (index != -1 && substring.indexOf(":") != -1) {
                    String substring2;
                    if (index == substring.length()) {
                        substring2 = "";
                    }
                    else {
                        substring = substring.substring(0, index + 1);
                        substring2 = substring.substring(index + 1);
                    }
                    sb.append(substring);
                    sb.append(quote(substring2, ParseUtil.L_REG_NAME | ParseUtil.L_SERVER, ParseUtil.H_REG_NAME | ParseUtil.H_SERVER));
                }
            }
            else {
                sb.append(quote(substring, ParseUtil.L_REG_NAME | ParseUtil.L_SERVER, ParseUtil.H_REG_NAME | ParseUtil.H_SERVER));
            }
        }
    }
    
    private static void appendFragment(StringBuffer sb, String s) {
        if (s != null) {
            sb.append('#');
            sb.append(quote(s, ParseUtil.L_URIC, ParseUtil.H_URIC));
        }
    }
    
    private static String quote(String s, long n, long n2) {
        s.length();
        Object o = null;
        int n3 = (n & 0x1L) != 0x0L;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 < '\u0080') {
                if (!match((char)char1, n, n2) && !isEscaped(s, i)) {
                    if (o == null) {
                        o = new StringBuffer();
                        o.append(s.substring(0, i));
                    }
                    appendEscape(o, (byte)char1);
                }
                else if (o != null) {
                    o.append((char)char1);
                }
            }
            else if (n3 && (Character.isSpaceChar((char)char1) || Character.isISOControl((char)char1))) {
                if (o == null) {
                    o = new StringBuffer();
                    o.append(s.substring(0, i));
                }
                appendEncoded(o, (char)char1);
            }
            else if (o != null) {
                o.append((char)char1);
            }
        }
        return (o == null) ? s : o.toString();
    }
    
    private static boolean isEscaped(String s, int index) {
        return s != null && s.length() > index + 2 && s.charAt(index) == '%' && match(s.charAt(index + 1), ParseUtil.L_HEX, ParseUtil.H_HEX) && match(s.charAt(index + 2), ParseUtil.L_HEX, ParseUtil.H_HEX);
    }
    
    private static void appendEncoded(StringBuffer sb, char c) {
        ByteBuffer encode = null;
        try {
            encode = ThreadLocalCoders.encoderFor("UTF-8").encode(CharBuffer.wrap("" + c));
        }
        catch (CharacterCodingException ex) {
            assert false;
        }
        while (encode.hasRemaining()) {
            int n = encode.get() & 0xFF;
            if (n >= 128) {
                appendEscape(sb, (byte)n);
            }
            else {
                sb.append((char)n);
            }
        }
    }
    
    private static void appendEscape(StringBuffer sb, byte b) {
        sb.append('%');
        sb.append(ParseUtil.hexDigits[b >> 4 & 0xF]);
        sb.append(ParseUtil.hexDigits[b >> 0 & 0xF]);
    }
    
    private static boolean match(char c, long n, long n2) {
        if (c < '@') {
            return (1L << c & n) != 0x0L;
        }
        return c < '\u0080' && (1L << c - 64 & n2) != 0x0L;
    }
    
    private static void checkPath(String input, String s, String s2) throws URISyntaxException {
        if (s != null && s2 != null && s2.length() > 0 && s2.charAt(0) != '/') {
            throw new URISyntaxException(input, "Relative path in absolute URI");
        }
        return;
    }
    
    private static long lowMask(char a, char a2) {
        long n = 0L;
        int i;
        int max;
        for (i = Math.max(Math.min((int)a, 63), 0), max = Math.max(Math.min((int)a2, 63), 0), ; i <= max; ++i) {
            n |= 1L << i;
        }
        return n;
    }
    
    private static long lowMask(String s) {
        int length = s.length();
        long n = 0L;
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (char1 < '@') {
                n |= 1L << char1;
            }
        }
        return n;
    }
    
    private static long highMask(char a, char a2) {
        long n = 0L;
        int i;
        int n2;
        for (i = Math.max(Math.min((int)a, 127), 64) - 64, n2 = Math.max(Math.min((int)a2, 127), 64) - 64, ; i <= n2; ++i) {
            n |= 1L << i;
        }
        return n;
    }
    
    private static long highMask(String s) {
        int length = s.length();
        long n = 0L;
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (char1 >= '@' && char1 < '\u0080') {
                n |= 1L << char1 - 64;
            }
        }
        return n;
    }
    
    static {
        (ParseUtil.encodedInPath = new BitSet(256)).set(61);
        ParseUtil.encodedInPath.set(59);
        ParseUtil.encodedInPath.set(63);
        ParseUtil.encodedInPath.set(47);
        ParseUtil.encodedInPath.set(35);
        ParseUtil.encodedInPath.set(32);
        ParseUtil.encodedInPath.set(60);
        ParseUtil.encodedInPath.set(62);
        ParseUtil.encodedInPath.set(37);
        ParseUtil.encodedInPath.set(34);
        ParseUtil.encodedInPath.set(123);
        ParseUtil.encodedInPath.set(125);
        ParseUtil.encodedInPath.set(124);
        ParseUtil.encodedInPath.set(92);
        ParseUtil.encodedInPath.set(94);
        ParseUtil.encodedInPath.set(91);
        ParseUtil.encodedInPath.set(93);
        ParseUtil.encodedInPath.set(96);
        for (int i = 0; i < 32; ++i) {
            ParseUtil.encodedInPath.set(i);
        }
        ParseUtil.encodedInPath.set(127);
        hexDigits = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        L_DIGIT = lowMask('0', '9');
        L_HEX = ParseUtil.L_DIGIT;
        H_HEX = (highMask('A', 'F') | highMask('a', 'f'));
        H_UPALPHA = highMask('A', 'Z');
        H_LOWALPHA = highMask('a', 'z');
        H_ALPHA = (ParseUtil.H_LOWALPHA | ParseUtil.H_UPALPHA);
        L_ALPHANUM = (ParseUtil.L_DIGIT | 0x0L);
        H_ALPHANUM = (0x0L | ParseUtil.H_ALPHA);
        L_MARK = lowMask("-_.!~*'()");
        H_MARK = highMask("-_.!~*'()");
        L_UNRESERVED = (ParseUtil.L_ALPHANUM | ParseUtil.L_MARK);
        H_UNRESERVED = (ParseUtil.H_ALPHANUM | ParseUtil.H_MARK);
        L_RESERVED = lowMask(";/?:@&=+$,[]");
        H_RESERVED = highMask(";/?:@&=+$,[]");
        L_DASH = lowMask("-");
        H_DASH = highMask("-");
        L_URIC = (ParseUtil.L_RESERVED | ParseUtil.L_UNRESERVED | 0x1L);
        H_URIC = (ParseUtil.H_RESERVED | ParseUtil.H_UNRESERVED | 0x0L);
        L_PCHAR = (ParseUtil.L_UNRESERVED | 0x1L | lowMask(":@&=+$,"));
        H_PCHAR = (ParseUtil.H_UNRESERVED | 0x0L | highMask(":@&=+$,"));
        L_PATH = (ParseUtil.L_PCHAR | lowMask(";/"));
        H_PATH = (ParseUtil.H_PCHAR | highMask(";/"));
        L_USERINFO = (ParseUtil.L_UNRESERVED | 0x1L | lowMask(";:&=+$,"));
        H_USERINFO = (ParseUtil.H_UNRESERVED | 0x0L | highMask(";:&=+$,"));
        L_REG_NAME = (ParseUtil.L_UNRESERVED | 0x1L | lowMask("$,;:@&=+"));
        H_REG_NAME = (ParseUtil.H_UNRESERVED | 0x0L | highMask("$,;:@&=+"));
        L_SERVER = (ParseUtil.L_USERINFO | ParseUtil.L_ALPHANUM | ParseUtil.L_DASH | lowMask(".:@[]"));
        H_SERVER = (ParseUtil.H_USERINFO | ParseUtil.H_ALPHANUM | ParseUtil.H_DASH | highMask(".:@[]"));
    }
}
